@@ -17,10 +17,7 @@
 #define _ONEDPL_GLUE_ASYNC_IMPL_H
 
 #include "../async_extension_defs.h"
-
-#if _ONEDPL_HETERO_BACKEND
-#    include "async_impl_hetero.h"
-#endif
+#include "async_impl_hetero.h"
 
 namespace oneapi
 {
@@ -28,6 +25,15 @@ namespace dpl
 {
 namespace experimental
 {
+// [wait_for_all]
+template <typename... _Ts>
+oneapi::dpl::__internal::__enable_if_convertible_to_events<void, _Ts...>
+wait_for_all(_Ts&&... __events)
+{
+    // TODO design a backend API function for waiting, and move this implementation into the SYCL backend
+    ::std::initializer_list<int> i = {0, (static_cast<sycl::event>(__events).wait_and_throw(), 0)...};
+    (void)i;
+}
 
 // [async.transform]
 template <class _ExecutionPolicy, class _ForwardIterator1, class _ForwardIterator2, class _UnaryOperation,
@@ -37,10 +43,12 @@ auto
 transform_async(_ExecutionPolicy&& __exec, _ForwardIterator1 __first, _ForwardIterator1 __last,
                 _ForwardIterator2 __result, _UnaryOperation __op, _Events&&... __dependencies)
 {
+    const auto __dispatch_tag = oneapi::dpl::__internal::__select_backend(__exec, __first, __result);
+
     wait_for_all(::std::forward<_Events>(__dependencies)...);
     auto ret_val = oneapi::dpl::__internal::__pattern_walk2_async(
-        ::std::forward<_ExecutionPolicy>(__exec), __first, __last, __result,
-        oneapi::dpl::__internal::__invoke_unary_op<_UnaryOperation>{::std::move(__op)});
+        __dispatch_tag, ::std::forward<_ExecutionPolicy>(__exec), __first, __last, __result,
+        oneapi::dpl::__internal::__transform_functor<_UnaryOperation>{::std::move(__op)});
     return ret_val;
 }
 
@@ -53,11 +61,12 @@ transform_async(_ExecutionPolicy&& __exec, _ForwardIterator1 __first1, _ForwardI
                 _ForwardIterator2 __first2, _ForwardIterator __result, _BinaryOperation __op,
                 _Events&&... __dependencies)
 {
+    const auto __dispatch_tag = oneapi::dpl::__internal::__select_backend(__exec, __first1, __first2, __result);
+
     wait_for_all(::std::forward<_Events>(__dependencies)...);
     auto ret_val = oneapi::dpl::__internal::__pattern_walk3_async(
-        ::std::forward<_ExecutionPolicy>(__exec), __first1, __last1, __first2, __result,
-        oneapi::dpl::__internal::__transform_functor<
-            oneapi::dpl::__internal::__ref_or_copy<_ExecutionPolicy, _BinaryOperation>>(__op));
+        __dispatch_tag, ::std::forward<_ExecutionPolicy>(__exec), __first1, __last1, __first2, __result,
+        oneapi::dpl::__internal::__transform_functor<_BinaryOperation>(::std::move(__op)));
     return ret_val;
 }
 
@@ -68,10 +77,12 @@ auto
 copy_async(_ExecutionPolicy&& __exec, _ForwardIterator1 __first, _ForwardIterator1 __last, _ForwardIterator2 __result,
            _Events&&... __dependencies)
 {
+    auto __dispatch_tag = oneapi::dpl::__internal::__select_backend(__exec, __first, __result);
+
     wait_for_all(::std::forward<_Events>(__dependencies)...);
     auto ret_val = oneapi::dpl::__internal::__pattern_walk2_brick_async(
-        ::std::forward<_ExecutionPolicy>(__exec), __first, __last, __result,
-        oneapi::dpl::__internal::__brick_copy<_ExecutionPolicy>{});
+        __dispatch_tag, ::std::forward<_ExecutionPolicy>(__exec), __first, __last, __result,
+        oneapi::dpl::__internal::__brick_copy<decltype(__dispatch_tag), _ExecutionPolicy>{});
     return ret_val;
 }
 
@@ -88,8 +99,22 @@ sort_async(_ExecutionPolicy&& __exec, _Iterator __first, _Iterator __last, _Comp
     auto __keep = oneapi::dpl::__ranges::__get_sycl_range<__par_backend_hetero::access_mode::read_write, _Iterator>();
     auto __buf = __keep(__first, __last);
 
-    return __par_backend_hetero::__parallel_stable_sort(::std::forward<_ExecutionPolicy>(__exec), __buf.all_view(),
-                                                        __comp);
+    const auto __dispatch_tag = oneapi::dpl::__internal::__select_backend(__exec, __first);
+    using __backend_tag = typename decltype(__dispatch_tag)::__backend_tag;
+
+    return __par_backend_hetero::__parallel_stable_sort(__backend_tag{}, ::std::forward<_ExecutionPolicy>(__exec),
+                                                        __buf.all_view(), __comp, oneapi::dpl::identity{});
+}
+
+template <class _ExecutionPolicy, class _RandomAccessIterator, class... _Events,
+          oneapi::dpl::__internal::__enable_if_device_execution_policy<_ExecutionPolicy, int, _Events...>>
+auto
+sort_async(_ExecutionPolicy&& __exec, _RandomAccessIterator __first, _RandomAccessIterator __last,
+           _Events&&... __dependencies)
+{
+    using _ValueType = typename ::std::iterator_traits<_RandomAccessIterator>::value_type;
+    return sort_async(::std::forward<_ExecutionPolicy>(__exec), __first, __last, ::std::less<_ValueType>(),
+                      ::std::forward<_Events>(__dependencies)...);
 }
 
 // [async.for_each]
@@ -99,9 +124,11 @@ auto
 for_each_async(_ExecutionPolicy&& __exec, _ForwardIterator __first, _ForwardIterator __last, _Function __f,
                _Events&&... __dependencies)
 {
+    const auto __dispatch_tag = oneapi::dpl::__internal::__select_backend(__exec, __first);
+
     wait_for_all(::std::forward<_Events>(__dependencies)...);
-    auto ret_val =
-        oneapi::dpl::__internal::__pattern_walk1_async(::std::forward<_ExecutionPolicy>(__exec), __first, __last, __f);
+    auto ret_val = oneapi::dpl::__internal::__pattern_walk1_async(
+        __dispatch_tag, ::std::forward<_ExecutionPolicy>(__exec), __first, __last, __f);
     return ret_val;
 }
 
@@ -114,14 +141,35 @@ auto
 reduce_async(_ExecutionPolicy&& __exec, _ForwardIterator __first, _ForwardIterator __last, _Tp __init,
              _BinaryOperation __binary_op, _Events&&... __dependencies)
 {
+    const auto __dispatch_tag = oneapi::dpl::__internal::__select_backend(__exec, __first);
+
     wait_for_all(::std::forward<_Events>(__dependencies)...);
-    typedef typename ::std::iterator_traits<_ForwardIterator>::value_type _InputType;
     auto ret_val = oneapi::dpl::__internal::__pattern_transform_reduce_async(
-        ::std::forward<_ExecutionPolicy>(__exec), __first, __last, __init, ::std::plus<_InputType>(),
+        __dispatch_tag, ::std::forward<_ExecutionPolicy>(__exec), __first, __last, __init, __binary_op,
         oneapi::dpl::__internal::__no_op());
     return ret_val;
 }
 
+template <class _ExecutionPolicy, class _ForwardIt, class... _Events,
+          oneapi::dpl::__internal::__enable_if_device_execution_policy<_ExecutionPolicy, int, _Events...>>
+auto
+reduce_async(_ExecutionPolicy&& __exec, _ForwardIt __first, _ForwardIt __last, _Events&&... __dependencies)
+{
+    using _ValueType = typename ::std::iterator_traits<_ForwardIt>::value_type;
+    return reduce_async(::std::forward<_ExecutionPolicy>(__exec), __first, __last, _ValueType(0),
+                        ::std::plus<_ValueType>(), ::std::forward<_Events>(__dependencies)...);
+}
+
+template <class _ExecutionPolicy, class _ForwardIt, class _T, class... _Events,
+          oneapi::dpl::__internal::__enable_if_device_execution_policy_single_no_default<_ExecutionPolicy, int, _T,
+                                                                                         _Events...>>
+auto
+reduce_async(_ExecutionPolicy&& __exec, _ForwardIt __first, _ForwardIt __last, _T __init, _Events&&... __dependencies)
+{
+    return reduce_async(::std::forward<_ExecutionPolicy>(__exec), __first, __last, __init, ::std::plus<_T>(),
+                        ::std::forward<_Events>(__dependencies)...);
+}
+
 // [async.fill]
 
 template <class _ExecutionPolicy, class _ForwardIterator, class _Tp, class... _Events,
@@ -130,9 +178,11 @@ auto
 fill_async(_ExecutionPolicy&& __exec, _ForwardIterator __first, _ForwardIterator __last, const _Tp& __value,
            _Events&&... __dependencies)
 {
+    const auto __dispatch_tag = oneapi::dpl::__internal::__select_backend(__exec, __first);
+
     wait_for_all(::std::forward<_Events>(__dependencies)...);
-    return oneapi::dpl::__internal::__pattern_fill_async(::std::forward<_ExecutionPolicy>(__exec), __first, __last,
-                                                         __value);
+    return oneapi::dpl::__internal::__pattern_fill_async(__dispatch_tag, ::std::forward<_ExecutionPolicy>(__exec),
+                                                         __first, __last, __value);
 }
 
 // [async.transform_reduce]
@@ -145,9 +195,12 @@ auto
 transform_reduce_async(_ExecutionPolicy&& __exec, _ForwardIt1 __first1, _ForwardIt1 __last1, _ForwardIt2 __first2,
                        _T __init, _BinaryOp1 __binary_op1, _BinaryOp2 __binary_op2, _Events&&... __dependencies)
 {
+    const auto __dispatch_tag = oneapi::dpl::__internal::__select_backend(__exec, __first1, __first2);
+
     wait_for_all(::std::forward<_Events>(__dependencies)...);
     return oneapi::dpl::__internal::__pattern_transform_reduce_async(
-        ::std::forward<_ExecutionPolicy>(__exec), __first1, __last1, __first2, __init, __binary_op1, __binary_op2);
+        __dispatch_tag, ::std::forward<_ExecutionPolicy>(__exec), __first1, __last1, __first2, __init, __binary_op1,
+        __binary_op2);
 }
 
 template <class _ExecutionPolicy, class _ForwardIt, class _T, class _BinaryOp, class _UnaryOp, class... _Events,
@@ -157,9 +210,23 @@ auto
 transform_reduce_async(_ExecutionPolicy&& __exec, _ForwardIt __first, _ForwardIt __last, _T __init,
                        _BinaryOp __binary_op, _UnaryOp __unary_op, _Events&&... __dependencies)
 {
+    const auto __dispatch_tag = oneapi::dpl::__internal::__select_backend(__exec, __first);
+
     wait_for_all(::std::forward<_Events>(__dependencies)...);
-    return oneapi::dpl::__internal::__pattern_transform_reduce_async(::std::forward<_ExecutionPolicy>(__exec), __first,
-                                                                     __last, __init, __binary_op, __unary_op);
+    return oneapi::dpl::__internal::__pattern_transform_reduce_async(
+        __dispatch_tag, ::std::forward<_ExecutionPolicy>(__exec), __first, __last, __init, __binary_op, __unary_op);
+}
+
+template <class _ExecutionPolicy, class _ForwardIt1, class _ForwardIt2, class _T, class... _Events,
+          oneapi::dpl::__internal::__enable_if_device_execution_policy<_ExecutionPolicy, int, _Events...>>
+auto
+transform_reduce_async(_ExecutionPolicy&& __exec, _ForwardIt1 __first1, _ForwardIt1 __last1, _ForwardIt2 __first2,
+                       _T __init, _Events&&... __dependencies)
+{
+    using _ValueType = typename ::std::iterator_traits<_ForwardIt1>::value_type;
+    return transform_reduce_async(::std::forward<_ExecutionPolicy>(__exec), __first1, __last1, __first2, __init,
+                                  ::std::plus<_T>(), ::std::multiplies<_ValueType>(),
+                                  ::std::forward<_Events>(__dependencies)...);
 }
 
 // [async.scan]
@@ -170,10 +237,13 @@ auto
 inclusive_scan_async(_ExecutionPolicy&& __exec, _ForwardIt1 __first1, _ForwardIt1 __last1, _ForwardIt2 __first2,
                      _Events&&... __dependencies)
 {
+    const auto __dispatch_tag = oneapi::dpl::__internal::__select_backend(__exec, __first1, __first2);
+
+    using _ValueType = typename ::std::iterator_traits<_ForwardIt1>::value_type;
     wait_for_all(::std::forward<_Events>(__dependencies)...);
     return oneapi::dpl::__internal::__pattern_transform_scan_async(
-        ::std::forward<_ExecutionPolicy>(__exec), __first1, __last1, __first2, oneapi::dpl::__internal::__no_op(),
-        ::std::plus<typename ::std::iterator_traits<_ForwardIt1>::value_type>(), /*inclusive=*/::std::true_type());
+        __dispatch_tag, ::std::forward<_ExecutionPolicy>(__exec), __first1, __last1, __first2,
+        oneapi::dpl::__internal::__no_op(), ::std::plus<_ValueType>(), /*inclusive=*/::std::true_type());
 }
 
 template <class _ExecutionPolicy, class _ForwardIt1, class _ForwardIt2, class _BinaryOperation, class... _Events,
@@ -183,10 +253,12 @@ auto
 inclusive_scan_async(_ExecutionPolicy&& __exec, _ForwardIt1 __first1, _ForwardIt1 __last1, _ForwardIt2 __first2,
                      _BinaryOperation __binary_op, _Events&&... __dependencies)
 {
+    const auto __dispatch_tag = oneapi::dpl::__internal::__select_backend(__exec, __first1, __first2);
+
     wait_for_all(::std::forward<_Events>(__dependencies)...);
     return oneapi::dpl::__internal::__pattern_transform_scan_async(
-        ::std::forward<_ExecutionPolicy>(__exec), __first1, __last1, __first2, oneapi::dpl::__internal::__no_op(),
-        __binary_op, /*inclusive=*/::std::true_type());
+        __dispatch_tag, ::std::forward<_ExecutionPolicy>(__exec), __first1, __last1, __first2,
+        oneapi::dpl::__internal::__no_op(), __binary_op, /*inclusive=*/::std::true_type());
 }
 
 template <class _ExecutionPolicy, class _ForwardIt1, class _ForwardIt2, class _BinaryOperation, class _T,
@@ -197,10 +269,12 @@ auto
 inclusive_scan_async(_ExecutionPolicy&& __exec, _ForwardIt1 __first1, _ForwardIt1 __last1, _ForwardIt2 __first2,
                      _BinaryOperation __binary_op, _T __init, _Events&&... __dependencies)
 {
+    const auto __dispatch_tag = oneapi::dpl::__internal::__select_backend(__exec, __first1, __first2);
+
     wait_for_all(::std::forward<_Events>(__dependencies)...);
     return oneapi::dpl::__internal::__pattern_transform_scan_async(
-        ::std::forward<_ExecutionPolicy>(__exec), __first1, __last1, __first2, oneapi::dpl::__internal::__no_op(),
-        __init, __binary_op, /*inclusive=*/::std::true_type());
+        __dispatch_tag, ::std::forward<_ExecutionPolicy>(__exec), __first1, __last1, __first2,
+        oneapi::dpl::__internal::__no_op(), __init, __binary_op, /*inclusive=*/::std::true_type());
 }
 
 template <class _ExecutionPolicy, class _ForwardIt1, class _ForwardIt2, class _T, class... _Events,
@@ -209,10 +283,12 @@ auto
 exclusive_scan_async(_ExecutionPolicy&& __exec, _ForwardIt1 __first1, _ForwardIt1 __last1, _ForwardIt2 __first2,
                      _T __init, _Events&&... __dependencies)
 {
+    const auto __dispatch_tag = oneapi::dpl::__internal::__select_backend(__exec, __first1, __first2);
+
     wait_for_all(::std::forward<_Events>(__dependencies)...);
     return oneapi::dpl::__internal::__pattern_transform_scan_async(
-        ::std::forward<_ExecutionPolicy>(__exec), __first1, __last1, __first2, oneapi::dpl::__internal::__no_op(),
-        __init, ::std::plus<_T>(), /*exclusive=*/::std::false_type());
+        __dispatch_tag, ::std::forward<_ExecutionPolicy>(__exec), __first1, __last1, __first2,
+        oneapi::dpl::__internal::__no_op(), __init, ::std::plus<_T>(), /*exclusive=*/::std::false_type());
 }
 
 template <class _ExecutionPolicy, class _ForwardIt1, class _ForwardIt2, class _T, class _BinaryOperation,
@@ -223,10 +299,12 @@ auto
 exclusive_scan_async(_ExecutionPolicy&& __exec, _ForwardIt1 __first1, _ForwardIt1 __last1, _ForwardIt2 __first2,
                      _T __init, _BinaryOperation __binary_op, _Events&&... __dependencies)
 {
+    const auto __dispatch_tag = oneapi::dpl::__internal::__select_backend(__exec, __first1, __first2);
+
     wait_for_all(::std::forward<_Events>(__dependencies)...);
     return oneapi::dpl::__internal::__pattern_transform_scan_async(
-        ::std::forward<_ExecutionPolicy>(__exec), __first1, __last1, __first2, oneapi::dpl::__internal::__no_op(),
-        __init, __binary_op, /*exclusive=*/::std::false_type());
+        __dispatch_tag, ::std::forward<_ExecutionPolicy>(__exec), __first1, __last1, __first2,
+        oneapi::dpl::__internal::__no_op(), __init, __binary_op, /*exclusive=*/::std::false_type());
 }
 
 template <class _ExecutionPolicy, class _ForwardIt1, class _ForwardIt2, class _T, class _BinaryOperation,
@@ -237,8 +315,11 @@ transform_exclusive_scan_async(_ExecutionPolicy&& __exec, _ForwardIt1 __first1,
                                _ForwardIt2 __first2, _T __init, _BinaryOperation __binary_op,
                                _UnaryOperation __unary_op, _Events&&... __dependencies)
 {
+    const auto __dispatch_tag = oneapi::dpl::__internal::__select_backend(__exec, __first1, __first2);
+
     wait_for_all(::std::forward<_Events>(__dependencies)...);
-    return oneapi::dpl::__internal::__pattern_transform_scan_async(::std::forward<_ExecutionPolicy>(__exec), __first1,
+    return oneapi::dpl::__internal::__pattern_transform_scan_async(__dispatch_tag,
+                                                                   ::std::forward<_ExecutionPolicy>(__exec), __first1,
                                                                    __last1, __first2, __unary_op, __init, __binary_op,
                                                                    /*exclusive=*/::std::false_type());
 }
@@ -251,10 +332,12 @@ transform_inclusive_scan_async(_ExecutionPolicy&& __exec, _ForwardIt1 __first1,
                                _ForwardIt2 __first2, _BinaryOperation __binary_op, _UnaryOperation __unary_op,
                                _Events&&... __dependencies)
 {
+    const auto __dispatch_tag = oneapi::dpl::__internal::__select_backend(__exec, __first1, __first2);
+
     wait_for_all(::std::forward<_Events>(__dependencies)...);
-    return oneapi::dpl::__internal::__pattern_transform_scan_async(::std::forward<_ExecutionPolicy>(__exec), __first1,
-                                                                   __last1, __first2, __unary_op, __binary_op,
-                                                                   /*inclusive=*/::std::true_type());
+    return oneapi::dpl::__internal::__pattern_transform_scan_async(
+        __dispatch_tag, ::std::forward<_ExecutionPolicy>(__exec), __first1, __last1, __first2, __unary_op, __binary_op,
+        /*inclusive=*/::std::true_type());
 }
 
 template <class _ExecutionPolicy, class _ForwardIt1, class _ForwardIt2, class _BinaryOperation, class _UnaryOperation,
@@ -266,8 +349,11 @@ transform_inclusive_scan_async(_ExecutionPolicy&& __exec, _ForwardIt1 __first1,
                                _ForwardIt2 __first2, _BinaryOperation __binary_op, _UnaryOperation __unary_op,
                                _T __init, _Events&&... __dependencies)
 {
+    const auto __dispatch_tag = oneapi::dpl::__internal::__select_backend(__exec, __first1, __first2);
+
     wait_for_all(::std::forward<_Events>(__dependencies)...);
-    return oneapi::dpl::__internal::__pattern_transform_scan_async(::std::forward<_ExecutionPolicy>(__exec), __first1,
+    return oneapi::dpl::__internal::__pattern_transform_scan_async(__dispatch_tag,
+                                                                   ::std::forward<_ExecutionPolicy>(__exec), __first1,
                                                                    __last1, __first2, __unary_op, __init, __binary_op,
                                                                    /*inclusive=*/::std::true_type());
 }
@@ -278,4 +364,4 @@ transform_inclusive_scan_async(_ExecutionPolicy&& __exec, _ForwardIt1 __first1,
 
 } // namespace oneapi
 
-#endif /* _ONEDPL_GLUE_ASYNC_IMPL_H */
+#endif // _ONEDPL_GLUE_ASYNC_IMPL_H
