@@ -19,11 +19,15 @@
 #ifndef _ONEDPL_INCLUSIVE_SCAN_BY_SEGMENT_IMPL_H
 #define _ONEDPL_INCLUSIVE_SCAN_BY_SEGMENT_IMPL_H
 
+#include "by_segment_extension_defs.h"
+
+#include "../pstl/glue_numeric_defs.h"
 #include "../pstl/glue_numeric_impl.h"
+
 #include "../pstl/parallel_backend.h"
 #include "function.h"
-#include "by_segment_extension_defs.h"
 #include "../pstl/utils.h"
+#include "scan_by_segment_impl.h"
 
 namespace oneapi
 {
@@ -35,12 +39,15 @@ namespace internal
 template <typename Name>
 class InclusiveScan1;
 
-template <typename Policy, typename InputIterator1, typename InputIterator2, typename OutputIterator,
+template <class _Tag, typename Policy, typename InputIterator1, typename InputIterator2, typename OutputIterator,
           typename BinaryPredicate, typename BinaryOperator>
-oneapi::dpl::__internal::__enable_if_host_execution_policy<typename ::std::decay<Policy>::type, OutputIterator>
-inclusive_scan_by_segment_impl(Policy&& policy, InputIterator1 first1, InputIterator1 last1, InputIterator2 first2,
-                               OutputIterator result, BinaryPredicate binary_pred, BinaryOperator binary_op)
+OutputIterator
+pattern_inclusive_scan_by_segment(_Tag, Policy&& policy, InputIterator1 first1, InputIterator1 last1,
+                                  InputIterator2 first2, OutputIterator result, BinaryPredicate binary_pred,
+                                  BinaryOperator binary_op)
 {
+    static_assert(__internal::__is_host_dispatch_tag_v<_Tag>);
+
     const auto n = ::std::distance(first1, last1);
 
     // Check for empty and single element ranges
@@ -54,34 +61,48 @@ inclusive_scan_by_segment_impl(Policy&& policy, InputIterator1 first1, InputIter
 
     typedef unsigned int FlagType;
     typedef typename ::std::iterator_traits<InputIterator2>::value_type ValueType;
-    typedef typename ::std::decay<Policy>::type policy_type;
 
-    oneapi::dpl::__par_backend::__buffer<policy_type, FlagType> _mask(n);
+    oneapi::dpl::__par_backend::__buffer<Policy, FlagType> _mask(policy, n);
     auto mask = _mask.get();
 
     mask[0] = 1;
 
-    transform(::std::forward<Policy>(policy), first1, last1 - 1, first1 + 1, _mask.get() + 1,
+    transform(policy, first1, last1 - 1, first1 + 1, _mask.get() + 1,
               oneapi::dpl::__internal::__not_pred<BinaryPredicate>(binary_pred));
 
-    typename internal::rebind_policy<policy_type, InclusiveScan1<policy_type>>::type policy1(policy);
-    inclusive_scan(policy1, make_zip_iterator(first2, _mask.get()), make_zip_iterator(first2, _mask.get()) + n,
-                   make_zip_iterator(result, _mask.get()),
+    inclusive_scan(::std::forward<Policy>(policy), make_zip_iterator(first2, _mask.get()),
+                   make_zip_iterator(first2, _mask.get()) + n, make_zip_iterator(result, _mask.get()),
                    internal::segmented_scan_fun<ValueType, FlagType, BinaryOperator>(binary_op));
 
     return result + n;
 }
 
 #if _ONEDPL_BACKEND_SYCL
-template <typename Policy, typename InputIterator1, typename InputIterator2, typename OutputIterator,
-          typename BinaryPredicate, typename BinaryOperator>
-oneapi::dpl::__internal::__enable_if_hetero_execution_policy<typename ::std::decay<Policy>::type, OutputIterator>
-inclusive_scan_by_segment_impl(Policy&& policy, InputIterator1 first1, InputIterator1 last1, InputIterator2 first2,
-                               OutputIterator result, BinaryPredicate binary_pred, BinaryOperator binary_op)
+template <typename _BackendTag, typename Policy, typename InputIterator1, typename InputIterator2,
+          typename OutputIterator, typename BinaryPredicate, typename BinaryOperator>
+OutputIterator
+inclusive_scan_by_segment_impl(__internal::__hetero_tag<_BackendTag> __tag, Policy&& policy, InputIterator1 first1,
+                               InputIterator1 last1, InputIterator2 first2, OutputIterator result,
+                               BinaryPredicate binary_pred, BinaryOperator binary_op,
+                               ::std::true_type /* has_known_identity */)
 {
+    using iter_value_t = typename ::std::iterator_traits<InputIterator2>::value_type;
+    iter_value_t identity = unseq_backend::__known_identity<BinaryOperator, iter_value_t>;
+    return internal::__scan_by_segment_impl_common(__tag, ::std::forward<Policy>(policy), first1, last1, first2, result,
+                                                   identity, binary_pred, binary_op, ::std::true_type{});
+}
+
+template <typename _BackendTag, typename Policy, typename InputIterator1, typename InputIterator2,
+          typename OutputIterator, typename BinaryPredicate, typename BinaryOperator>
+OutputIterator
+inclusive_scan_by_segment_impl(__internal::__hetero_tag<_BackendTag>, Policy&& policy, InputIterator1 first1,
+                               InputIterator1 last1, InputIterator2 first2, OutputIterator result,
+                               BinaryPredicate binary_pred, BinaryOperator binary_op,
+                               ::std::false_type /* has_known_identity */)
+{
+
     typedef unsigned int FlagType;
     typedef typename ::std::iterator_traits<InputIterator2>::value_type ValueType;
-    typedef typename ::std::decay<Policy>::type policy_type;
 
     const auto n = ::std::distance(first1, last1);
 
@@ -91,10 +112,10 @@ inclusive_scan_by_segment_impl(Policy&& policy, InputIterator1 first1, InputIter
 
     FlagType initial_mask = 1;
 
-    internal::__buffer<policy_type, FlagType> _mask(policy, n);
+    oneapi::dpl::__par_backend_hetero::__buffer<Policy, FlagType> _mask(policy, n);
     {
         auto mask_buf = _mask.get_buffer();
-        auto mask = mask_buf.template get_access<sycl::access::mode::read_write>();
+        auto mask = mask_buf.get_host_access(sycl::read_write);
 
         mask[0] = initial_mask;
     }
@@ -102,14 +123,27 @@ inclusive_scan_by_segment_impl(Policy&& policy, InputIterator1 first1, InputIter
     transform(::std::forward<Policy>(policy), first1, last1 - 1, first1 + 1, _mask.get() + 1,
               oneapi::dpl::__internal::__not_pred<BinaryPredicate>(binary_pred));
 
-    typename internal::rebind_policy<policy_type, InclusiveScan1<policy_type>>::type policy1(policy);
-    transform_inclusive_scan(policy1, make_zip_iterator(first2, _mask.get()),
+    auto policy1 = oneapi::dpl::__par_backend_hetero::make_wrapped_policy<InclusiveScan1>(policy);
+    transform_inclusive_scan(::std::move(policy1), make_zip_iterator(first2, _mask.get()),
                              make_zip_iterator(first2, _mask.get()) + n, make_zip_iterator(result, _mask.get()),
                              internal::segmented_scan_fun<ValueType, FlagType, BinaryOperator>(binary_op),
                              oneapi::dpl::__internal::__no_op());
-
     return result + n;
 }
+
+template <typename _BackendTag, typename Policy, typename InputIterator1, typename InputIterator2,
+          typename OutputIterator, typename BinaryPredicate, typename BinaryOperator>
+OutputIterator
+pattern_inclusive_scan_by_segment(__internal::__hetero_tag<_BackendTag> __tag, Policy&& policy, InputIterator1 first1,
+                                  InputIterator1 last1, InputIterator2 first2, OutputIterator result,
+                                  BinaryPredicate binary_pred, BinaryOperator binary_op)
+{
+    return internal::inclusive_scan_by_segment_impl(
+        __tag, ::std::forward<Policy>(policy), first1, last1, first2, result, binary_pred, binary_op,
+        typename unseq_backend::__has_known_identity<
+            BinaryOperator, typename ::std::iterator_traits<InputIterator2>::value_type>::type{});
+}
+
 #endif
 } // namespace internal
 
@@ -119,8 +153,10 @@ oneapi::dpl::__internal::__enable_if_execution_policy<Policy, OutputIterator>
 inclusive_scan_by_segment(Policy&& policy, InputIterator1 first1, InputIterator1 last1, InputIterator2 first2,
                           OutputIterator result, BinaryPredicate binary_pred, BinaryOperator binary_op)
 {
-    return internal::inclusive_scan_by_segment_impl(::std::forward<Policy>(policy), first1, last1, first2, result,
-                                                    binary_pred, binary_op);
+    const auto __dispatch_tag = oneapi::dpl::__internal::__select_backend(policy, first1, first2, result);
+
+    return internal::pattern_inclusive_scan_by_segment(__dispatch_tag, ::std::forward<Policy>(policy), first1, last1,
+                                                       first2, result, binary_pred, binary_op);
 }
 
 template <typename Policy, typename InputIter1, typename InputIter2, typename OutputIter, typename BinaryPredicate>
@@ -171,4 +207,4 @@ inclusive_scan_by_key(Policy&& policy, InputIter1 first1, InputIter1 last1, Inpu
 } // end namespace dpl
 } // end namespace oneapi
 
-#endif
+#endif // _ONEDPL_INCLUSIVE_SCAN_BY_SEGMENT_IMPL_H
