@@ -16,50 +16,56 @@
 #ifndef _ONEDPL_CONFIG_H
 #define _ONEDPL_CONFIG_H
 
-#ifndef _PSTL_VERSION
-#    define _PSTL_VERSION 11000
-#    define _PSTL_VERSION_MAJOR (_PSTL_VERSION / 1000)
-#    define _PSTL_VERSION_MINOR ((_PSTL_VERSION % 1000) / 10)
-#    define _PSTL_VERSION_PATCH (_PSTL_VERSION % 10)
+// The version header also defines a few configuration macros used in this file
+#include "../internal/version_impl.h"
+
+// -- Check availability of parallel backends --
+
+#if __has_include(<tbb/tbb.h>)
+#    define _ONEDPL_TBB_AVAILABLE 1
+#endif
+#if ONEDPL_USE_TBB_BACKEND && !_ONEDPL_TBB_AVAILABLE
+#    error "Parallel execution policies with oneTBB or Intel(R) TBB support are enabled, but the library is not found"
 #endif
 
-#define ONEDPL_VERSION_MAJOR 2022
-#define ONEDPL_VERSION_MINOR 0
-#define ONEDPL_VERSION_PATCH 0
-
-#if defined(ONEDPL_USE_DPCPP_BACKEND)
-#    undef _ONEDPL_BACKEND_SYCL
-#    define _ONEDPL_BACKEND_SYCL ONEDPL_USE_DPCPP_BACKEND
+#if defined(_OPENMP) && __has_include(<omp.h>)
+#    define _ONEDPL_OPENMP_AVAILABLE 1
+#endif
+// During compilation for a device _OPENMP may not be set, so avoid throwing an error if
+// __SYCL_DEVICE_ONLY__ found
+#if ONEDPL_USE_OPENMP_BACKEND && !_ONEDPL_OPENMP_AVAILABLE && !defined(__SYCL_DEVICE_ONLY__)
+#    error "Parallel execution policies with OpenMP* support are enabled, \
+        but OpenMP headers are not found or the compiler does not support OpenMP"
 #endif
 
-#if defined(ONEDPL_FPGA_DEVICE)
-#    undef _ONEDPL_FPGA_DEVICE
-#    define _ONEDPL_FPGA_DEVICE ONEDPL_FPGA_DEVICE
+// -- Check availability of heterogeneous backends --
+
+// If DPCPP backend is explicitly requested, optimistically assume SYCL availability;
+// otherwise, make sure that it is definitely available additionally checking SYCL_LANGUAGE_VERSION
+#if __has_include(<sycl/sycl.hpp>) || __has_include(<CL/sycl.hpp>)
+#    if SYCL_LANGUAGE_VERSION || CL_SYCL_LANGUAGE_VERSION || ONEDPL_USE_DPCPP_BACKEND
+#        define _ONEDPL_SYCL_AVAILABLE 1
+#    endif
+#else
+#    if ONEDPL_USE_DPCPP_BACKEND
+#        error "Device execution policies are requested, but SYCL* headers are not found"
+#    endif
 #endif
 
-#if defined(ONEDPL_FPGA_EMULATOR)
-#    undef _ONEDPL_FPGA_EMU
-#    define _ONEDPL_FPGA_EMU ONEDPL_FPGA_EMULATOR
+// If DPCPP backend is not explicitly turned off and SYCL is available, enable it
+#if (ONEDPL_USE_DPCPP_BACKEND || !defined(ONEDPL_USE_DPCPP_BACKEND)) && _ONEDPL_SYCL_AVAILABLE
+#    define _ONEDPL_BACKEND_SYCL 1
 #endif
 
-#if defined(ONEDPL_USE_PREDEFINED_POLICIES)
-#    undef _ONEDPL_PREDEFINED_POLICIES
-#    define _ONEDPL_PREDEFINED_POLICIES ONEDPL_USE_PREDEFINED_POLICIES
-#elif !defined(_ONEDPL_PREDEFINED_POLICIES)
-#    define _ONEDPL_PREDEFINED_POLICIES 1
+// If at least one heterogeneous backend is available, enable them
+#if _ONEDPL_BACKEND_SYCL
+#    if _ONEDPL_HETERO_BACKEND
+#        undef _ONEDPL_HETERO_BACKEND
+#    endif
+#    define _ONEDPL_HETERO_BACKEND 1
 #endif
 
-#if ONEDPL_USE_TBB_BACKEND || (!defined(ONEDPL_USE_TBB_BACKEND) && !ONEDPL_USE_OPENMP_BACKEND)
-#    define _ONEDPL_PAR_BACKEND_TBB 1
-#endif
-
-#if ONEDPL_USE_OPENMP_BACKEND || (!defined(ONEDPL_USE_OPENMP_BACKEND) && defined(_OPENMP))
-#    define _ONEDPL_PAR_BACKEND_OPENMP 1
-#endif
-
-#if !_ONEDPL_PAR_BACKEND_TBB && !_ONEDPL_PAR_BACKEND_OPENMP
-#    define _ONEDPL_PAR_BACKEND_SERIAL 1
-#endif
+// -- Configure host backends and common parts --
 
 // Check the user-defined macro for warnings
 #if !defined(_PSTL_USAGE_WARNINGS) && defined(PSTL_USAGE_WARNINGS)
@@ -90,7 +96,8 @@
 #endif
 
 // Enable SIMD for compilers that support OpenMP 4.0
-#if (_OPENMP >= 201307) || (__INTEL_COMPILER >= 1600) || (!defined(__INTEL_COMPILER) && _ONEDPL_GCC_VERSION >= 40900)
+#if (_OPENMP >= 201307) || __INTEL_LLVM_COMPILER || (__INTEL_COMPILER >= 1600) ||                                      \
+    (!defined(__INTEL_LLVM_COMPILER) && !defined(__INTEL_COMPILER) && _ONEDPL_GCC_VERSION >= 40900)
 #    define _ONEDPL_PRAGMA_SIMD _ONEDPL_PRAGMA(omp simd)
 #    define _ONEDPL_PRAGMA_DECLARE_SIMD _ONEDPL_PRAGMA(omp declare simd)
 #    define _ONEDPL_PRAGMA_SIMD_REDUCTION(PRM) _ONEDPL_PRAGMA(omp simd reduction(PRM))
@@ -104,7 +111,16 @@
 #    define _ONEDPL_PRAGMA_SIMD_REDUCTION(PRM)
 #endif //Enable SIMD
 
-#if (__INTEL_COMPILER)
+// Enable loop unrolling pragmas where supported
+#if (__INTEL_LLVM_COMPILER || __INTEL_COMPILER ||                                                                      \
+     (!defined(__INTEL_LLVM_COMPILER) && !defined(__INTEL_COMPILER) &&                                                 \
+      ((_ONEDPL_GCC_VERSION >= 80000) || (_ONEDPL_CLANG_VERSION >= 30700))))
+#    define _ONEDPL_PRAGMA_UNROLL _ONEDPL_PRAGMA(unroll)
+#else //no pragma unroll
+#    define _ONEDPL_PRAGMA_UNROLL
+#endif
+
+#if (__INTEL_LLVM_COMPILER || __INTEL_COMPILER)
 #    define _ONEDPL_PRAGMA_FORCEINLINE _ONEDPL_PRAGMA(forceinline)
 #elif defined(_PSTL_PRAGMA_FORCEINLINE)
 #    define _ONEDPL_PRAGMA_FORCEINLINE _PSTL_PRAGMA_FORCEINLINE
@@ -112,7 +128,7 @@
 #    define _ONEDPL_PRAGMA_FORCEINLINE
 #endif
 
-#if (__INTEL_COMPILER >= 1900)
+#if (__INTEL_LLVM_COMPILER >= 20230100 || __INTEL_COMPILER >= 1900)
 #    define _ONEDPL_PRAGMA_SIMD_SCAN(PRM) _ONEDPL_PRAGMA(omp simd reduction(inscan, PRM))
 #    define _ONEDPL_PRAGMA_SIMD_INCLUSIVE_SCAN(PRM) _ONEDPL_PRAGMA(omp scan inclusive(PRM))
 #    define _ONEDPL_PRAGMA_SIMD_EXCLUSIVE_SCAN(PRM) _ONEDPL_PRAGMA(omp scan exclusive(PRM))
@@ -139,8 +155,16 @@
 
 // Should be defined to 1 for environments with a vendor implementation of C++17 execution policies
 #define _ONEDPL_CPP17_EXECUTION_POLICIES_PRESENT                                                                       \
-    (_MSC_VER >= 1912 && _MSVC_LANG >= 201703L) ||                                                                     \
-        (_GLIBCXX_RELEASE >= 9 && __GLIBCXX__ >= 20190503 && __cplusplus >= 201703L)
+    (_ONEDPL___cplusplus >= 201703L && (_MSC_VER >= 1912 || (_GLIBCXX_RELEASE >= 9 && __GLIBCXX__ >= 20190503)))
+
+// In the SYCL backend reduce-then-scan path, we need to be able to differentiate between when a compiler enables
+// optimizations and when it does not. With GCC and clang-based compilers, we can detect this with the __OPTIMIZE__
+// flag.
+#if _ONEDPL_GCC_VERSION > 0 || defined(_ONEDPL_CLANG_VERSION)
+#    define _ONEDPL_DETECT_COMPILER_OPTIMIZATIONS_ENABLED __OPTIMIZE__
+#else
+#    define _ONEDPL_DETECT_COMPILER_OPTIMIZATIONS_ENABLED 0
+#endif
 
 #define _ONEDPL_EARLYEXIT_PRESENT (__INTEL_COMPILER >= 1800)
 #if (defined(_PSTL_PRAGMA_SIMD_EARLYEXIT) && _PSTL_EARLYEXIT_PRESENT)
@@ -165,13 +189,20 @@
 #    define _ONEDPL_PRAGMA_SIMD_ORDERED_MONOTONIC_2ARGS(PRM1, PRM2)
 #endif
 
-#if (__INTEL_COMPILER >= 1900 || !defined(__INTEL_COMPILER) && _ONEDPL_GCC_VERSION >= 40900 || _OPENMP >= 201307)
+#if (_OPENMP >= 201307 || __INTEL_LLVM_COMPILER || __INTEL_COMPILER >= 1900 ||                                         \
+     !defined(__INTEL_LLVM_COMPILER) && !defined(__INTEL_COMPILER) && _ONEDPL_GCC_VERSION >= 40900)
 #    define _ONEDPL_UDR_PRESENT 1
 #else
 #    define _ONEDPL_UDR_PRESENT 0
 #endif
 
-#define _ONEDPL_UDS_PRESENT (__INTEL_COMPILER >= 1900 && __INTEL_COMPILER_BUILD_DATE >= 20180626)
+// TODO: enable UDS on Windows with Intel LLVM-based compiler when it is fixed
+#if (__INTEL_LLVM_COMPILER >= 20230100 && !defined(_MSC_VER)) ||                                                       \
+    (__INTEL_COMPILER >= 1900 && __INTEL_COMPILER_BUILD_DATE >= 20180626)
+#    define _ONEDPL_UDS_PRESENT 1
+#else
+#    define _ONEDPL_UDS_PRESENT 0
+#endif
 
 // Declaration of reduction functor, where
 // NAME - the name of the functor
@@ -197,7 +228,7 @@
 
 // Check the user-defined macro to use non-temporal stores
 #ifndef _PSTL_USE_NONTEMPORAL_STORES_IF_ALLOWED
-#    if defined(PSTL_USE_NONTEMPORAL_STORES) && (__INTEL_COMPILER >= 1600)
+#    if defined(PSTL_USE_NONTEMPORAL_STORES) && (__INTEL_LLVM_COMPILER || __INTEL_COMPILER >= 1600)
 #        define _PSTL_USE_NONTEMPORAL_STORES_IF_ALLOWED _PSTL_PRAGMA(vector nontemporal)
 #    else
 #        define _PSTL_USE_NONTEMPORAL_STORES_IF_ALLOWED
@@ -227,61 +258,92 @@
 // Some  C++ standard libraries contain 'exclusive_scan' declaration (version with binary_op)
 // w/o "enable_if". So, a call 'exclusive_scan' may be ambiguous in case of a custom policy using.
 #define _ONEDPL_EXCLUSIVE_SCAN_WITH_BINARY_OP_AMBIGUITY                                                                \
-    (__GLIBCXX__ && __GLIBCXX__ > 20190503 && __cplusplus >= 201703L)
+    (_ONEDPL___cplusplus >= 201703L && __GLIBCXX__ && __GLIBCXX__ > 20190503)
 
 // some algorithms in <numeric> such as 'reduce' were added since libstdc++-9.3, we
 // have to provide our own implementation if legacy libstdc++ is in use.
 #define _ONEDPL_HAS_NUMERIC_SERIAL_IMPL                                                                                \
     (__GLIBCXX__ && (_GLIBCXX_RELEASE < 9 || (_GLIBCXX_RELEASE == 9 && __GLIBCXX__ < 20200312)))
 
-// Check the user-defined macro for parallel policies
-// define _ONEDPL_BACKEND_SYCL 1 when we compile with the Compiler that supports SYCL
-#if !defined(_ONEDPL_BACKEND_SYCL)
-#    if (defined(CL_SYCL_LANGUAGE_VERSION) || defined(SYCL_LANGUAGE_VERSION))
-#        define _ONEDPL_BACKEND_SYCL 1
-#    else
-#        define _ONEDPL_BACKEND_SYCL 0
-#    endif // CL_SYCL_LANGUAGE_VERSION
+//'present' macros
+// shift_left, shift_right; GCC 10; VS 2019 16.1
+#define _ONEDPL_CPP20_SHIFT_LEFT_RIGHT_PRESENT                                                                         \
+    (_ONEDPL___cplusplus >= 202002L && (_MSC_VER >= 1921 || _GLIBCXX_RELEASE >= 10))
+
+#if _ONEDPL_STD_FEATURE_MACROS_PRESENT
+#    define _ONEDPL_CPP20_CONCEPTS_PRESENT (__cpp_concepts >= 201907L && __cpp_lib_concepts >= 202002L)
+#    define _ONEDPL_CPP23_TUPLE_LIKE_COMMON_REFERENCE_PRESENT                                                          \
+        (_ONEDPL___cplusplus >= 202302L && __cpp_lib_tuple_like >= 202207L)
+#    define _ONEDPL_CPP23_RANGES_ZIP_PRESENT (_ONEDPL___cplusplus >= 202302L && __cpp_lib_ranges_zip >= 202110L)
+#else
+#    define _ONEDPL_CPP20_CONCEPTS_PRESENT 0
+#    define _ONEDPL_CPP23_TUPLE_LIKE_COMMON_REFERENCE_PRESENT 0
+#    define _ONEDPL_CPP23_RANGES_ZIP_PRESENT 0
 #endif
 
-// if SYCL policy switch on then let's switch hetero policy macro on
-#if _ONEDPL_BACKEND_SYCL
-#    if _ONEDPL_HETERO_BACKEND
-#        undef _ONEDPL_HETERO_BACKEND
-#    endif
-#    define _ONEDPL_HETERO_BACKEND 1
-// Include sycl specific options
-// FPGA doesn't support sub-groups
-#    if !(_ONEDPL_FPGA_DEVICE)
-#        define _USE_SUB_GROUPS 1
-#        define _USE_GROUP_ALGOS 1
-#    endif
+// When C++20 concepts are available, we must use std::tuple as a proxy reference to satisfy iterator concepts, which
+// requires the changes to std::tuple in P2321R2 and the tuple-like basic_common_reference specialization in P2165R4.
+#define _ONEDPL_CAN_USE_STD_TUPLE_PROXY_ITERATOR                                                                       \
+    (!_ONEDPL_CPP20_CONCEPTS_PRESENT ||                                                                                \
+     (_ONEDPL_CPP23_RANGES_ZIP_PRESENT && _ONEDPL_CPP23_TUPLE_LIKE_COMMON_REFERENCE_PRESENT))
 
-#    define _USE_RADIX_SORT (_USE_SUB_GROUPS && _USE_GROUP_ALGOS)
-
-// Compilation of a kernel is requiried to obtain valid work_group_size
-// when target devices are CPU or FPGA emulator. Since CPU and GPU devices
-// cannot be distinguished during compilation, the macro is enabled by default.
-#    if !defined(_ONEDPL_COMPILE_KERNEL)
-#        define _ONEDPL_COMPILE_KERNEL 1
-#    endif
+#if _ONEDPL_CPP20_CONCEPTS_PRESENT
+#    define _ONEDPL_CPP20_REQUIRES(req) requires(req)
+#else
+#    define _ONEDPL_CPP20_REQUIRES(req)
 #endif
 
+#if defined(_MSC_VER) && __INTEL_LLVM_COMPILER < 20240100
+#    define _ONEDPL_ICPX_OMP_SIMD_DESTROY_WINDOWS_BROKEN 1
+#else
+#    define _ONEDPL_ICPX_OMP_SIMD_DESTROY_WINDOWS_BROKEN 0
+#endif
+
+// The implementation of std::bit_floor in MS STL does not meet requirements for SYCL device functions
+#if defined(_MSC_VER) && (__SYCL_DEVICE_ONLY__ || __SYCL_SINGLE_SOURCE__)
+#    define _ONEDPL_STD_BIT_FLOOR_BROKEN 1
+#else
+#    define _ONEDPL_STD_BIT_FLOOR_BROKEN 0
+#endif
+
+// The implementation of std::ranges algorithms in MS C++ standard library is done via C++ functions.
+#if defined(_MSC_VER) && (_MSC_VER < 1939)
+#    define _ONEDPL_STD_RANGES_ALGO_CPP_FUN 1
+#else
+#    define _ONEDPL_STD_RANGES_ALGO_CPP_FUN 0
+#endif
+
+// -- Configure heterogeneous backends --
+
 #if !defined(ONEDPL_ALLOW_DEFERRED_WAITING)
 #    define ONEDPL_ALLOW_DEFERRED_WAITING 0
 #endif
 
-//'present' macros
-// shift_left, shift_right; GCC 10; VS 2019 16.1
-#define _ONEDPL_CPP20_SHIFT_LEFT_RIGHT_PRESENT                                                                         \
-    (__cplusplus >= 202002L && ((_GLIBCXX_RELEASE >= 10) || (_MSC_VER >= 1921)))
-
-#define _ONEDPL_BUILT_IN_STABLE_NAME_PRESENT __has_builtin(__builtin_sycl_unique_stable_name)
-
-// When compile with Intel(R) oneAPI DPC++ Compiler macro is on
-// We need this macro to use compiler specific implementation of SYCL
-#if defined(__INTEL_LLVM_COMPILER) && defined(SYCL_LANGUAGE_VERSION)
-#    define _ONEDPL_SYCL_INTEL_COMPILER 1
+#if defined(ONEDPL_USE_PREDEFINED_POLICIES)
+#    undef _ONEDPL_PREDEFINED_POLICIES
+#    define _ONEDPL_PREDEFINED_POLICIES ONEDPL_USE_PREDEFINED_POLICIES
+#elif !defined(_ONEDPL_PREDEFINED_POLICIES)
+#    define _ONEDPL_PREDEFINED_POLICIES 1
 #endif
 
-#endif /* _ONEDPL_CONFIG_H */
+#if defined(ONEDPL_FPGA_DEVICE)
+#    undef _ONEDPL_FPGA_DEVICE
+#    define _ONEDPL_FPGA_DEVICE ONEDPL_FPGA_DEVICE
+#endif
+#if defined(ONEDPL_FPGA_EMULATOR)
+#    undef _ONEDPL_FPGA_EMU
+#    define _ONEDPL_FPGA_EMU ONEDPL_FPGA_EMULATOR
+#endif
+
+#if _ONEDPL_BACKEND_SYCL
+// Include sycl specific options
+// FPGA doesn't support sub-groups
+#    if !(_ONEDPL_FPGA_DEVICE)
+#        define _ONEDPL_USE_SUB_GROUPS 1
+#        define _ONEDPL_USE_GROUP_ALGOS 1
+#    endif
+
+#    define _ONEDPL_BUILT_IN_STABLE_NAME_PRESENT __has_builtin(__builtin_sycl_unique_stable_name)
+#endif // _ONEDPL_BACKEND_SYCL
+
+#endif // _ONEDPL_CONFIG_H
