@@ -16,17 +16,106 @@
 #ifndef _ONEDPL_UTILS_RANGES_H
 #define _ONEDPL_UTILS_RANGES_H
 
-#include <iterator>
-#include <type_traits>
+#include <tuple>       // std::get
+#include <cstdint>     // std::uint8_t
+#include <cstddef>     // std::size_t, std::ptrdiff_t
+#include <utility>     // std::declval
+#include <iterator>    // std::iterator_traits
+#include <type_traits> // std::decay_t, std::remove_cv_t, std::remove_reference_t, std::invoke_result_t, ...
+
+#if _ONEDPL_CPP20_RANGES_PRESENT
+#    include <ranges> // std::ranges::sized_range, std::ranges::range_size_t
+#endif
 
 #include "iterator_defs.h"
 #include "iterator_impl.h"
-#include "execution_defs.h"
 
 namespace oneapi
 {
 namespace dpl
 {
+
+namespace __internal
+{
+
+template <typename _R>
+auto
+get_value_type(int) -> typename ::std::decay_t<_R>::value_type;
+
+template <typename _R>
+auto
+get_value_type(long) ->
+    typename ::std::iterator_traits<::std::decay_t<decltype(::std::declval<_R&>().begin())>>::value_type;
+
+template <typename _It>
+auto
+get_value_type(long long) -> typename ::std::iterator_traits<_It>::value_type;
+
+template <typename _R>
+auto
+get_value_type(...)
+{
+    //static_assert should always fail when this overload is chosen, so its condition must depend on
+    //the template parameter and evaluate to false
+    static_assert(std::is_same_v<_R, void>,
+        "error: the range has no 'value_type'; define an alias or typedef named 'value_type' in the range class");
+}
+
+template <typename _R>
+using __value_t = decltype(oneapi::dpl::__internal::get_value_type<_R>(0));
+
+template <typename _Proj, typename _R>
+using __key_t = ::std::remove_cv_t<::std::remove_reference_t<::std::invoke_result_t<_Proj&, __value_t<_R>>>>;
+
+template <typename T, typename = void>
+struct __range_has_raw_ptr_iterator : ::std::false_type
+{
+};
+
+template <typename T>
+struct __range_has_raw_ptr_iterator<T, ::std::void_t<decltype(::std::declval<T&>().begin())>>
+    : ::std::is_pointer<decltype(::std::declval<T&>().begin())>
+{
+};
+
+template <typename T>
+inline constexpr bool __range_has_raw_ptr_iterator_v = __range_has_raw_ptr_iterator<T>::value;
+
+#if _ONEDPL_CPP20_RANGES_PRESENT
+//The following '__range_size' type trait should be used in only the context with std::common_type
+//together with a sized range.
+template <typename R>
+struct __range_size {
+    using type = std::uint8_t;
+};
+
+template <std::ranges::sized_range R>
+struct __range_size<R> {
+    using type = std::ranges::range_size_t<R>;
+};
+
+template <typename _R>
+using __range_size_t = typename __range_size<_R>::type;
+
+#endif //_ONEDPL_CPP20_RANGES_PRESENT
+
+template <typename _R>
+auto
+__check_size(int) -> decltype(std::declval<_R&>().size());
+
+template <typename _R>
+auto
+__check_size(long) -> decltype(std::declval<_R&>().get_count());
+
+template <typename _It>
+auto
+__check_size(...) -> typename std::iterator_traits<_It>::difference_type;
+
+template <typename _R>
+using __difference_t = std::make_signed_t<decltype(__check_size<_R>(0))>;
+
+} //namespace __internal
+
 namespace __ranges
 {
 
@@ -53,9 +142,9 @@ struct pipeline_base
 };
 
 template <typename Range>
-struct pipeline_base<Range, typename ::std::enable_if<is_pipeline_object<Range>::value>::type>
+struct pipeline_base<Range, ::std::enable_if_t<is_pipeline_object<Range>::value>>
 {
-    using type = typename pipeline_base<typename ::std::decay<decltype(::std::declval<Range>().base())>::type>::type;
+    using type = typename pipeline_base<::std::decay_t<decltype(::std::declval<Range>().base())>>::type;
 };
 
 //pipeline_base_range
@@ -74,7 +163,7 @@ struct pipeline_base_range
 
 // use ::std::conditional to understand what class to inherit from
 template <typename Range>
-struct pipeline_base_range<Range, typename ::std::enable_if<is_pipeline_object<Range>::value, void>::type>
+struct pipeline_base_range<Range, ::std::enable_if_t<is_pipeline_object<Range>::value>>
 {
     Range rng;
 
@@ -111,7 +200,7 @@ class zip_view
     }
 
   public:
-    using __value_t = oneapi::dpl::__internal::tuple<oneapi::dpl::__internal::__value_t<_Ranges>...>;
+    using value_type = oneapi::dpl::__internal::tuple<oneapi::dpl::__internal::__value_t<_Ranges>...>;
     static constexpr ::std::size_t __num_ranges = sizeof...(_Ranges);
 
     explicit zip_view(_Ranges... __args) : __m_ranges(oneapi::dpl::__internal::make_tuple(__args...)) {}
@@ -162,11 +251,11 @@ make_zip_view(_Ranges&&... args) -> decltype(zip_view<_Ranges...>(::std::forward
 template <typename _Iterator>
 class guard_view
 {
-    using value_type = typename ::std::iterator_traits<_Iterator>::value_type;
-    using reference = typename ::std::iterator_traits<_Iterator>::reference;
     using diff_type = typename ::std::iterator_traits<_Iterator>::difference_type;
 
   public:
+    using value_type = typename ::std::iterator_traits<_Iterator>::value_type;
+
     guard_view(_Iterator __first = _Iterator(), diff_type __n = 0) : m_p(__first), m_count(__n) {}
     guard_view(_Iterator __first, _Iterator __last) : m_p(__first), m_count(__last - __first) {}
 
@@ -209,8 +298,12 @@ class guard_view
 template <typename _R>
 struct reverse_view_simple
 {
+    using value_type = oneapi::dpl::__internal::__value_t<_R>;
+
     _R __r;
 
+    reverse_view_simple(_R __rng) : __r(__rng) {}
+
     //TODO: to be consistent with C++ standard, this Idx should be changed to diff_type of underlying range
     template <typename Idx>
     auto operator[](Idx __i) const -> decltype(__r[__i])
@@ -237,14 +330,17 @@ struct reverse_view_simple
     }
 };
 
-//It is kind of pseudo-view for take_view support.
+//It is kind of pseudo-view for take_view support. We assume that the underlying range will not shrink
+//after creation of the view to favor performance.
 template <typename _R, typename _Size>
 struct take_view_simple
 {
+    using value_type = oneapi::dpl::__internal::__value_t<_R>;
+
     _R __r;
     _Size __n;
 
-    take_view_simple(_R __rng, _Size __size) : __r(__rng), __n(__size) { assert(__n >= 0 && __n < __r.size()); }
+    take_view_simple(_R __rng, _Size __size) : __r(__rng), __n(__size) { assert(__n >= 0 && __n <= __r.size()); }
 
     //TODO: to be consistent with C++ standard, this Idx should be changed to diff_type of underlying range
     template <typename Idx>
@@ -272,14 +368,17 @@ struct take_view_simple
     }
 };
 
-//It is kind of pseudo-view for drop_view support.
+//It is kind of pseudo-view for drop_view support. We assume that the underlying range will not shrink
+//after creation of the view to favor performance.
 template <typename _R, typename _Size>
 struct drop_view_simple
 {
+    using value_type = oneapi::dpl::__internal::__value_t<_R>;
+
     _R __r;
     _Size __n;
 
-    drop_view_simple(_R __rng, _Size __size) : __r(__rng), __n(__size) { assert(__n >= 0 && __n < __r.size()); }
+    drop_view_simple(_R __rng, _Size __size) : __r(__rng), __n(__size) { assert(__n >= 0 && __n <= __r.size()); }
 
     //TODO: to be consistent with C++ standard, this Idx should be changed to diff_type of underlying range
     template <typename Idx>
@@ -307,10 +406,54 @@ struct drop_view_simple
     }
 };
 
+//replicate_start_view_simple inserts replicates of the first element m times, then continues with the range as normal.
+// For counting iterator range {0,1,2,3,4,5,...}, and __replicate_count = 3, the result is {0,0,0,0,1,2,3,4,5,...}
+template <typename _R, typename _Size>
+struct replicate_start_view_simple
+{
+    using value_type = oneapi::dpl::__internal::__value_t<_R>;
+
+    _R __r;
+    _Size __repl_count;
+
+    replicate_start_view_simple(_R __rng, _Size __replicate_count) : __r(__rng), __repl_count(__replicate_count)
+    {
+        assert(__repl_count >= 0);
+    }
+
+    //TODO: to be consistent with C++ standard, this Idx should be changed to diff_type of underlying range
+    template <typename Idx>
+    auto operator[](Idx __i) const -> decltype(__r[__i])
+    {
+        return (__i < __repl_count) ? __r[0] : __r[__i - __repl_count];
+    }
+
+    _Size
+    size() const
+    {
+        // if base range is empty, replication does not extend the valid size
+        return (__r.empty()) ? 0 : __r.size() + __repl_count;
+    }
+
+    bool
+    empty() const
+    {
+        return size() == 0;
+    }
+
+    auto
+    base() const -> decltype(__r)
+    {
+        return __r;
+    }
+};
+
 //It is kind of pseudo-view for transfom_iterator support.
 template <typename _R, typename _F>
 struct transform_view_simple
 {
+    using value_type = ::std::decay_t<::std::invoke_result_t<_F&, decltype(::std::declval<_R&>()[0])>>;
+
     _R __r;
     _F __f;
 
@@ -342,7 +485,7 @@ struct transform_view_simple
 
 template <typename _Map>
 auto
-test_map_view(int) -> decltype(::std::declval<_Map>().begin(), ::std::true_type{});
+test_map_view(int) -> decltype(::std::declval<_Map>()[0], ::std::true_type{});
 
 template <typename _Map>
 auto
@@ -355,30 +498,33 @@ struct is_map_view : decltype(test_map_view<_Map>(0))
 };
 
 //It is kind of pseudo-view for permutation_iterator support.
-template <typename _R, typename _M, typename = void>
+template <typename _Source, typename _M, typename = void>
 struct permutation_view_simple;
 
 //permutation view: specialization for an index map functor
-template <typename _R, typename _M>
-struct permutation_view_simple<_R, _M, typename ::std::enable_if<oneapi::dpl::__internal::__is_functor<_M>>::type>
+//size of such view  is specified by a caller
+template <typename _Source, typename _M>
+struct permutation_view_simple<_Source, _M, ::std::enable_if_t<oneapi::dpl::__internal::__is_functor<_M>>>
 {
-    using _Size = oneapi::dpl::__internal::__difference_t<_R>;
+    using value_type = oneapi::dpl::__internal::__value_t<_Source>;
+    using _Size = oneapi::dpl::__internal::__difference_t<_Source>;
 
-    _R __r;
+    _Source __src; //Iterator (pointer) or unreachable range
     _M __map_fn;
     _Size __size;
 
-    permutation_view_simple(_R __rng, _M __m, _Size __s) : __r(__rng), __map_fn(__m), __size(__s) {}
+    permutation_view_simple(_Source __data, _M __m, _Size __s) : __src(__data), __map_fn(__m), __size(__s) {}
 
     //TODO: to be consistent with C++ standard, this Idx should be changed to diff_type of underlying range
     template <typename Idx>
-    auto operator[](Idx __i) const -> decltype(__r[__map_fn(__i)])
+    decltype(auto)
+    operator[](Idx __i) const
     {
-        return __r[__map_fn(__i)];
+        return __src[__map_fn(__i)];
     }
 
     auto
-    size() const -> decltype(__size)
+    size() const
     {
         return __size;
     }
@@ -390,31 +536,36 @@ struct permutation_view_simple<_R, _M, typename ::std::enable_if<oneapi::dpl::__
     }
 
     auto
-    base() const -> decltype(__r)
+    base() const
     {
-        return __r;
+        return __src;
     }
 };
 
 //permutation view: specialization for a map view (a viewable range concept)
-template <typename _R, typename _M>
-struct permutation_view_simple<_R, _M, typename ::std::enable_if<is_map_view<_M>::value>::type>
+//size of such view  is specified by size of the map view (permutation range)
+template <typename _Source, typename _M>
+struct permutation_view_simple<_Source, _M, ::std::enable_if_t<is_map_view<_M>::value>>
 {
-    zip_view<_R, _M> __data;
+    using value_type = oneapi::dpl::__internal::__value_t<_Source>;
 
-    permutation_view_simple(_R __r, _M __m) : __data(__r, __m) {}
+    _Source __src; //Iterator (pointer) or unreachable range
+    _M __map;      //permutation range
+
+    permutation_view_simple(_Source __data, _M __m) : __src(__data), __map(__m) {}
 
     //TODO: to be consistent with C++ standard, this Idx should be changed to diff_type of underlying range
     template <typename Idx>
-    auto operator[](Idx __i) const -> decltype(::std::get<0>(__data.tuple())[::std::get<1>(__data.tuple())[__i]])
+    decltype(auto)
+    operator[](Idx __i) const
     {
-        return ::std::get<0>(__data.tuple())[::std::get<1>(__data.tuple())[__i]];
+        return __src[__map[__i]];
     }
 
     auto
-    size() const -> decltype(::std::get<1>(__data.tuple()).size())
+    size() const
     {
-        return ::std::get<1>(__data.tuple()).size();
+        return __map.size();
     }
 
     bool
@@ -424,15 +575,16 @@ struct permutation_view_simple<_R, _M, typename ::std::enable_if<is_map_view<_M>
     }
 
     auto
-    base() const -> decltype(__data)
+    base() const
     {
-        return __data;
+        return oneapi::dpl::__internal::make_tuple(__src, __map);
     }
 };
 
 //permutation discard view:
 struct permutation_discard_view
 {
+    using value_type = oneapi::dpl::internal::ignore_copyable;
     using difference_type = ::std::ptrdiff_t;
     difference_type m_count;
 
@@ -454,16 +606,7 @@ struct permutation_discard_view
 };
 
 } // namespace __ranges
-
-namespace __internal
-{
-template <typename... _Ranges>
-struct __range_traits<oneapi::dpl::__ranges::zip_view<_Ranges...>>
-{
-    using __value_t = typename oneapi::dpl::__ranges::zip_view<_Ranges...>::__value_t;
-};
-} // namespace __internal
 } // namespace dpl
 } // namespace oneapi
 
-#endif /* _ONEDPL_UTILS_RANGES_H */
+#endif // _ONEDPL_UTILS_RANGES_H
