@@ -37,16 +37,20 @@ test_device_copyable()
                   "noop_device_copyable is not trivially copy constructible");
     static_assert(!std::is_trivially_copy_constructible_v<constant_iterator_device_copyable>,
                   "constant_iterator_device_copyable is not trivially copy constructible");
+    static_assert(!std::is_trivially_copy_constructible_v<range_device_copyable>,
+                  "range_device_copyable is not trivially copy constructible");
 
     static_assert(sycl::is_device_copyable_v<int_device_copyable>, "int_device_copyable is not device copyable");
     static_assert(sycl::is_device_copyable_v<noop_device_copyable>, "noop_device_copyable is not device copyable");
     static_assert(sycl::is_device_copyable_v<constant_iterator_device_copyable>,
                   "constant_iterator_device_copyable is not device copyable");
 
-    //custom_brick
-    static_assert(sycl::is_device_copyable_v<oneapi::dpl::internal::custom_brick<
-                      noop_device_copyable, int_device_copyable, oneapi::dpl::internal::search_algorithm::lower_bound>>,
-                  "custom_brick is not device copyable with device copyable types");
+    //__custom_brick
+    static_assert(
+        sycl::is_device_copyable_v<
+            oneapi::dpl::internal::__custom_brick<noop_device_copyable, int_device_copyable, range_device_copyable,
+                                                  oneapi::dpl::internal::search_algorithm::lower_bound>>,
+        "__custom_brick is not device copyable with device copyable types");
     //replace_if_fun
     static_assert(
         sycl::is_device_copyable_v<oneapi::dpl::internal::replace_if_fun<int_device_copyable, noop_device_copyable>>,
@@ -74,11 +78,23 @@ test_device_copyable()
     static_assert(sycl::is_device_copyable_v<
                       oneapi::dpl::unseq_backend::walk_n<policy_non_device_copyable, noop_device_copyable>>,
                   "walk_n is not device copyable with device copyable types");
+    //walk1_vector_or_scalar
+    static_assert(sycl::is_device_copyable_v<oneapi::dpl::unseq_backend::walk1_vector_or_scalar<
+                      policy_non_device_copyable, noop_device_copyable, range_device_copyable>>,
+                  "walk1_vector_or_scalar is not device copyable with device copyable types");
+    //walk2_vectors_or_scalars
+    static_assert(sycl::is_device_copyable_v<oneapi::dpl::unseq_backend::walk2_vectors_or_scalars<
+                      policy_non_device_copyable, noop_device_copyable, range_device_copyable, range_device_copyable>>,
+                  "walk2_vectors_or_scalars is not device copyable with device copyable types");
+    //walk3_vectors_or_scalars
+    static_assert(sycl::is_device_copyable_v<oneapi::dpl::unseq_backend::walk3_vectors_or_scalars<
+                      policy_non_device_copyable, noop_device_copyable, range_device_copyable, range_device_copyable,
+                      range_device_copyable>>,
+                  "walk3_vectors_or_scalars is not device copyable with device copyable types");
     //walk_adjacent_difference
-    static_assert(
-        sycl::is_device_copyable_v<
-            oneapi::dpl::unseq_backend::walk_adjacent_difference<policy_non_device_copyable, noop_device_copyable>>,
-        "walk_adjacent_difference is not device copyable with device copyable types");
+    static_assert(sycl::is_device_copyable_v<oneapi::dpl::unseq_backend::walk_adjacent_difference<
+                      policy_non_device_copyable, noop_device_copyable, range_device_copyable, range_device_copyable>>,
+                  "walk_adjacent_difference is not device copyable with device copyable types");
     //transform_reduce
     static_assert(
         sycl::is_device_copyable_v<
@@ -94,6 +110,11 @@ test_device_copyable()
         sycl::is_device_copyable_v<
             oneapi::dpl::unseq_backend::single_match_pred_by_idx<policy_non_device_copyable, noop_device_copyable>>,
         "single_match_pred_by_idx is not device copyable with device copyable types");
+    //single_match_pred
+    static_assert(
+        sycl::is_device_copyable_v<
+            oneapi::dpl::unseq_backend::single_match_pred<policy_non_device_copyable, noop_device_copyable>>,
+        "single_match_pred is not device copyable with device copyable types");
     //multiple_match_pred
     static_assert(
         sycl::is_device_copyable_v<
@@ -141,8 +162,8 @@ test_device_copyable()
                                                        int_device_copyable, int_device_copyable, std::true_type>>,
         "__brick_set_op is not device copyable with device copyable types");
     // __brick_reduce_idx
-    static_assert(sycl::is_device_copyable_v<
-                      oneapi::dpl::unseq_backend::__brick_reduce_idx<noop_device_copyable, int_device_copyable>>,
+    static_assert(sycl::is_device_copyable_v<oneapi::dpl::unseq_backend::__brick_reduce_idx<
+                      noop_device_copyable, int_device_copyable, range_device_copyable>>,
                   "__brick_reduce_idx is not device copyable with device copyable types");
 
     //__gen_transform_input
@@ -150,6 +171,16 @@ test_device_copyable()
         sycl::is_device_copyable_v<oneapi::dpl::__par_backend_hetero::__gen_transform_input<noop_device_copyable>>,
         "__gen_transform_input is not device copyable with device copyable types");
 
+    //__gen_red_by_seg_reduce_input
+    static_assert(sycl::is_device_copyable_v<
+                      oneapi::dpl::__par_backend_hetero::__gen_red_by_seg_reduce_input<binary_op_device_copyable>>,
+                  "__gen_red_by_seg_reduce_input is not device copyable with device copyable types");
+
+    //__gen_red_by_seg_scan_input
+    static_assert(sycl::is_device_copyable_v<
+                      oneapi::dpl::__par_backend_hetero::__gen_red_by_seg_scan_input<binary_op_device_copyable>>,
+                  "__gen_red_by_seg_scan_input is not device copyable with device copyable types");
+
     //__gen_mask
     static_assert(sycl::is_device_copyable_v<oneapi::dpl::__par_backend_hetero::__gen_mask<noop_device_copyable>>,
                   "__gen_mask is not device copyable with device copyable types");
@@ -179,6 +210,11 @@ test_device_copyable()
         sycl::is_device_copyable_v<oneapi::dpl::__par_backend_hetero::__write_to_id_if_else<assign_device_copyable>>,
         "__write_to_id_if_else is not device copyable with device copyable types");
 
+    //__write_red_by_seg
+    static_assert(
+        sycl::is_device_copyable_v<oneapi::dpl::__par_backend_hetero::__write_red_by_seg<binary_op_device_copyable>>,
+        "__write_red_by_seg is not device copyable with device copyable types");
+
     // __early_exit_find_or
     static_assert(
         sycl::is_device_copyable_v<
@@ -190,10 +226,15 @@ test_device_copyable()
     // sycl::is_device_copyable specialization for __leaf_sorter does not require instantiation of
     // __leaf_sorter with the provided types. See [temp.inst]/1 of C++17 spec for the details.
     static_assert(
-        sycl::is_device_copyable_v<oneapi::dpl::__par_backend_hetero::__leaf_sorter<noop_device_copyable,
+        sycl::is_device_copyable_v<oneapi::dpl::__par_backend_hetero::__leaf_sorter<range_device_copyable,
                                                                                     noop_device_copyable>>,
         "__leaf_sorter is not device copyable with device copyable types");
 
+    //__red_by_seg_op
+    static_assert(
+        sycl::is_device_copyable_v<oneapi::dpl::__par_backend_hetero::__red_by_seg_op<binary_op_device_copyable>>,
+        "__red_by_seg_op is not device copyable with device copyable types");
+
     //__not_pred
     static_assert(sycl::is_device_copyable_v<oneapi::dpl::__internal::__not_pred<noop_device_copyable>>,
                   "__not_pred is not device copyable with device copyable types");
@@ -261,6 +302,10 @@ test_device_copyable()
     static_assert(sycl::is_device_copyable_v<
                       oneapi::dpl::__internal::__create_mask_unique_copy<noop_device_copyable, int_device_copyable>>,
                   "__create_mask_unique_copy is not device copyable with device copyable types");
+    static_assert(sycl::is_device_copyable_v<
+                      oneapi::dpl::__internal::__create_mask_unique_copy<noop_device_copyable,
+                      int_non_device_copyable>>,
+                  "__create_mask_unique_copy is incorrectly not device copyable because of non member field template arg");
     //tuple
     static_assert(sycl::is_device_copyable_v<oneapi::dpl::__internal::tuple<int_device_copyable, int_device_copyable>>,
                   "tuple is not device copyable with device copyable types");
@@ -277,12 +322,13 @@ test_non_device_copyable()
     static_assert(!sycl::is_device_copyable_v<noop_non_device_copyable>, "functor is device copyable");
     static_assert(!sycl::is_device_copyable_v<int_non_device_copyable>, "struct is device copyable");
     static_assert(!sycl::is_device_copyable_v<constant_iterator_non_device_copyable>, "iterator is device copyable");
+    static_assert(!sycl::is_device_copyable_v<range_non_device_copyable>, "range_non_device_copyable is device copyable");
 
-    //custom_brick
-    static_assert(
-        !sycl::is_device_copyable_v<oneapi::dpl::internal::custom_brick<
-            noop_device_copyable, int_non_device_copyable, oneapi::dpl::internal::search_algorithm::lower_bound>>,
-        "custom_brick is device copyable with non device copyable types");
+    //__custom_brick
+    static_assert(!sycl::is_device_copyable_v<oneapi::dpl::internal::__custom_brick<
+                      noop_device_copyable, int_non_device_copyable, range_non_device_copyable,
+                      oneapi::dpl::internal::search_algorithm::lower_bound>>,
+                  "__custom_brick is device copyable with non device copyable types");
     //replace_if_fun
     static_assert(!sycl::is_device_copyable_v<
                       oneapi::dpl::internal::replace_if_fun<int_device_copyable, noop_non_device_copyable>>,
@@ -311,10 +357,26 @@ test_non_device_copyable()
     static_assert(!sycl::is_device_copyable_v<
                       oneapi::dpl::unseq_backend::walk_n<policy_non_device_copyable, noop_non_device_copyable>>,
                   "walk_n is device copyable with non device copyable types");
+    //walk1_vector_or_scalar
+    static_assert(!sycl::is_device_copyable_v<oneapi::dpl::unseq_backend::walk1_vector_or_scalar<
+                      policy_non_device_copyable, noop_non_device_copyable, range_non_device_copyable>>,
+                  "walk1_vector_or_scalar is device copyable with non device copyable types");
+    //walk2_vectors_or_scalars
+    static_assert(
+        !sycl::is_device_copyable_v<
+            oneapi::dpl::unseq_backend::walk2_vectors_or_scalars<policy_non_device_copyable, noop_non_device_copyable,
+                                                                 range_non_device_copyable, range_non_device_copyable>>,
+        "walk2_vectors_or_scalars is device copyable with non device copyable types");
+    //walk3_vectors_or_scalars
+    static_assert(!sycl::is_device_copyable_v<oneapi::dpl::unseq_backend::walk3_vectors_or_scalars<
+                      policy_non_device_copyable, noop_non_device_copyable, range_non_device_copyable,
+                      range_non_device_copyable, range_non_device_copyable>>,
+                  "walk3_vectors_or_scalars is device copyable with non device copyable types");
     //walk_adjacent_difference
     static_assert(
         !sycl::is_device_copyable_v<
-            oneapi::dpl::unseq_backend::walk_adjacent_difference<policy_non_device_copyable, noop_non_device_copyable>>,
+            oneapi::dpl::unseq_backend::walk_adjacent_difference<policy_non_device_copyable, noop_non_device_copyable,
+                                                                 range_non_device_copyable, range_non_device_copyable>>,
         "walk_adjacent_difference is device copyable with non device copyable types");
     //transform_reduce
     static_assert(
@@ -331,6 +393,11 @@ test_non_device_copyable()
         !sycl::is_device_copyable_v<
             oneapi::dpl::unseq_backend::single_match_pred_by_idx<policy_non_device_copyable, noop_non_device_copyable>>,
         "single_match_pred_by_idx is device copyable with non device copyable types");
+    //single_match_pred
+    static_assert(
+        !sycl::is_device_copyable_v<
+            oneapi::dpl::unseq_backend::single_match_pred<policy_non_device_copyable, noop_non_device_copyable>>,
+        "single_match_pred is device copyable with non device copyable types");
     //multiple_match_pred
     static_assert(
         !sycl::is_device_copyable_v<
@@ -378,8 +445,8 @@ test_non_device_copyable()
                                                        int_device_copyable, int_device_copyable, std::true_type>>,
         "__brick_set_op is device copyable with non device copyable types");
     //__brick_reduce_idx
-    static_assert(!sycl::is_device_copyable_v<
-                      oneapi::dpl::unseq_backend::__brick_reduce_idx<noop_device_copyable, int_non_device_copyable>>,
+    static_assert(!sycl::is_device_copyable_v<oneapi::dpl::unseq_backend::__brick_reduce_idx<
+                      noop_device_copyable, int_non_device_copyable, range_non_device_copyable>>,
                   "__brick_reduce_idx is device copyable with non device copyable types");
 
     //__gen_transform_input
@@ -387,6 +454,16 @@ test_non_device_copyable()
         !sycl::is_device_copyable_v<oneapi::dpl::__par_backend_hetero::__gen_transform_input<noop_non_device_copyable>>,
         "__gen_transform_input is device copyable with non device copyable types");
 
+    //__gen_red_by_seg_reduce_input
+    static_assert(!sycl::is_device_copyable_v<
+                      oneapi::dpl::__par_backend_hetero::__gen_red_by_seg_reduce_input<binary_op_non_device_copyable>>,
+                  "__gen_red_by_seg_reduce_input is device copyable with non device copyable types");
+
+    //__gen_red_by_seg_reduce_input
+    static_assert(!sycl::is_device_copyable_v<
+                      oneapi::dpl::__par_backend_hetero::__gen_red_by_seg_scan_input<binary_op_non_device_copyable>>,
+                  "__gen_red_by_seg_scan_input is device copyable with non device copyable types");
+
     //__gen_mask
     static_assert(!sycl::is_device_copyable_v<oneapi::dpl::__par_backend_hetero::__gen_mask<noop_non_device_copyable>>,
                   "__gen_mask is device copyable with non device copyable types");
@@ -416,6 +493,11 @@ test_non_device_copyable()
                       oneapi::dpl::__par_backend_hetero::__write_to_id_if_else<assign_non_device_copyable>>,
                   "__write_to_id_if_else is device copyable with non device copyable types");
 
+    //__write_red_by_seg
+    static_assert(!sycl::is_device_copyable_v<
+                      oneapi::dpl::__par_backend_hetero::__write_red_by_seg<binary_op_non_device_copyable>>,
+                  "__write_red_by_seg is device copyable with non device copyable types");
+
     // __early_exit_find_or
     static_assert(
         !sycl::is_device_copyable_v<oneapi::dpl::__par_backend_hetero::__early_exit_find_or<policy_non_device_copyable,
@@ -423,10 +505,15 @@ test_non_device_copyable()
         "__early_exit_find_or is device copyable with non device copyable types");
 
     // __leaf_sorter
-    static_assert(!sycl::is_device_copyable_v<oneapi::dpl::__par_backend_hetero::__leaf_sorter<noop_device_copyable,
+    static_assert(!sycl::is_device_copyable_v<oneapi::dpl::__par_backend_hetero::__leaf_sorter<range_non_device_copyable,
                                                                                                noop_non_device_copyable>>,
                   "__leaf_sorter is device copyable with non device copyable types");
 
+    //__red_by_seg_op
+    static_assert(
+        !sycl::is_device_copyable_v<oneapi::dpl::__par_backend_hetero::__red_by_seg_op<binary_op_non_device_copyable>>,
+        "__red_by_seg_op is device copyable with non device copyable types");
+
     //__not_pred
     static_assert(!sycl::is_device_copyable_v<oneapi::dpl::__internal::__not_pred<noop_non_device_copyable>>,
                   "__not_pred is device copyable with non device copyable types");
