@@ -101,10 +101,12 @@ int
 main()
 {
 
-    const ::std::size_t N = 100000;
+    const std::size_t N = TestUtils::get_pattern_for_max_n();
 
     for (::std::size_t n = 0; n < N; n = n < 16 ? n + 1 : size_t(3.1415 * n))
     {
+        test_fill_by_type<std::int8_t>(n);
+        test_fill_by_type<std::int16_t>(n);
         test_fill_by_type<std::int32_t>(n);
         test_fill_by_type<float64_t>(n);
     }
