@@ -115,7 +115,7 @@ template <typename T>
 void
 test()
 {
-    const ::std::size_t max_len = 100000;
+    const std::size_t max_len = TestUtils::get_pattern_for_max_n();
 
     Sequence<T> data(max_len);
     Sequence<T> actual(max_len);
@@ -132,6 +132,7 @@ main()
 {
     test<wrapper<std::uint16_t>>();
     test<wrapper<float32_t>>();
+    test<std::uint8_t>();
     test<std::int32_t>();
     test<float64_t>();
 
