@@ -13,10 +13,10 @@
 //
 //===----------------------------------------------------------------------===//
 
-#include <oneapi/dpl/execution>
-
 #include "support/test_config.h"
 
+#include <oneapi/dpl/execution>
+
 #if _ENABLE_RANGES_TESTING
 #include <oneapi/dpl/ranges>
 #endif
@@ -54,7 +54,8 @@ main()
     auto expected1 = ::std::inner_product(data, data + max_n, data, 0);
     auto expected2 = ::std::inner_product(data, data + max_n, data, 0, ::std::plus<int>(), ::std::multiplies<int>());
 
-    auto data_view = nano::views::all(data) | oneapi::dpl::experimental::ranges::views::transform(lambda1);
+    //the name nano::ranges::views::all is not injected into oneapi::dpl::experimental::ranges namespace
+    auto data_view = __nanorange::nano::views::all(data) | oneapi::dpl::experimental::ranges::views::transform(lambda1);
     auto expected3 = ::std::accumulate(data_view.begin(), data_view.end(), 0);
 
     EXPECT_TRUE(res1 == expected1, "wrong effect from transform_reduce1 with sycl ranges");
