@@ -17,25 +17,25 @@ void
 test()
 {
     dpl::complex<T> c;
-    const dpl::complex<T> c2(1.5, 2.5);
+    const dpl::complex<T> c2(1.5f, 2.5f);
     assert(c.real() == 0);
     assert(c.imag() == 0);
     c += c2;
-    assert(c.real() == 1.5);
-    assert(c.imag() == 2.5);
+    assert(c.real() == 1.5f);
+    assert(c.imag() == 2.5f);
     c += c2;
     assert(c.real() == 3);
     assert(c.imag() == 5);
 
     dpl::complex<T> c3;
 
-#if !_PSTL_GLIBCXX_TEST_COMPLEX_PLUS_EQ_BROKEN
+#if _PSTL_TEST_COMPLEX_NON_FLOAT_AVAILABLE && !_PSTL_GLIBCXX_TEST_COMPLEX_PLUS_EQ_BROKEN
     c3 = c;
     dpl::complex<int> ic (1,1);
     c3 += ic;
     assert(c3.real() == 4);
     assert(c3.imag() == 6);
-#endif // !_PSTL_GLIBCXX_TEST_COMPLEX_PLUS_EQ_BROKEN
+#endif // _PSTL_TEST_COMPLEX_NON_FLOAT_AVAILABLE && !_PSTL_GLIBCXX_TEST_COMPLEX_PLUS_EQ_BROKEN
 
     c3 = c;
     dpl::complex<float> fc (1,1);
